@@ -18,6 +18,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "config.h" // For GLOM_ENABLE_CLIENT_ONLY
+
 #include "base_db.h"
 #include "application.h" //App_Glom.
 #include <glom/libglom/appstate.h>
@@ -25,10 +27,14 @@
 #include <glom/libglom/document/document_glom.h>
 #include <glom/libglom/data_structure/glomconversions.h>
 #include "mode_data/dialog_choose_field.h"
+
+//#ifndef GLOM_ENABLE_CLIENT_ONLY
 #include "layout_item_dialogs/dialog_field_layout.h"
 #include "layout_item_dialogs/dialog_notebook.h"
 #include "layout_item_dialogs/dialog_textobject.h"
 #include "layout_item_dialogs/dialog_imageobject.h"
+//#endif // !GLOM_ENABLE_CLIENT_ONLY
+
 //#include "reports/dialog_layout_report.h"
 #include <glom/libglom/utils.h>
 #include <glom/libglom/data_structure/glomconversions.h>
@@ -43,6 +49,10 @@
 #include <glibmm/i18n.h>
 //#include <libgnomeui/gnome-app-helper.h>
 
+#ifdef GLOM_ENABLE_MAEMO
+#include <hildonmm/note.h>
+#endif
+
 #include <sstream> //For stringstream
 
 namespace Glom
@@ -56,8 +66,8 @@ FoundSet::FoundSet(const FoundSet& src)
 :  m_table_name(src.m_table_name),
    m_extra_join(src.m_extra_join),
    m_where_clause(src.m_where_clause),
-   m_sort_clause(src.m_sort_clause),
-   m_extra_group_by(src.m_extra_group_by)
+   m_extra_group_by(src.m_extra_group_by),
+   m_sort_clause(src.m_sort_clause)
 {
 }
 
@@ -66,8 +76,8 @@ FoundSet& FoundSet::operator=(const FoundSet& src)
   m_table_name = src.m_table_name;
   m_extra_join = src.m_extra_join;
   m_where_clause = src.m_where_clause;
-  m_sort_clause = src.m_sort_clause;
   m_extra_group_by = src.m_extra_group_by;
+  m_sort_clause = src.m_sort_clause;
 
   return *this;
 }
@@ -77,8 +87,8 @@ bool FoundSet::operator==(const FoundSet& src) const
   return (m_table_name == src.m_table_name)
       && (m_extra_join == src.m_extra_join)
       && (m_where_clause == src.m_where_clause)
-      && (m_sort_clause == src.m_sort_clause)
-      && (m_extra_group_by == src.m_extra_group_by);
+      && (m_extra_group_by == src.m_extra_group_by)
+      && (m_sort_clause == src.m_sort_clause);
 }
 
 
@@ -147,14 +157,22 @@ void Base_DB::fill_end()
 }
 
 //static:
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
 sharedptr<SharedConnection> Base_DB::connect_to_server(Gtk::Window* parent_window)
+#else
+sharedptr<SharedConnection> Base_DB::connect_to_server(Gtk::Window* parent_window, std::auto_ptr<ExceptionConnection>& error)
+#endif
 {
   Bakery::BusyCursor busy_cursor(parent_window);
 
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   return ConnectionPool::get_and_connect();
+#else
+  return ConnectionPool::get_and_connect(error);
+#endif
 }
 
-void Base_DB::handle_error(const std::exception& ex)
+void Base_DB::handle_error(const Glib::Exception& ex)
 {
   std::cerr << "Internal Error (Base_DB::handle_error()): exception type=" << typeid(ex).name() << ", ex.what()=" << ex.what() << std::endl;
 
@@ -164,6 +182,20 @@ void Base_DB::handle_error(const std::exception& ex)
   dialog.run();
 }
 
+void Base_DB::handle_error(const std::exception& ex)
+{
+  std::cerr << "Internal Error (Base_DB::handle_error()): exception type=" << typeid(ex).name() << ", ex.what()=" << ex.what() << std::endl;
+
+#ifdef GLOM_ENABLE_MAEMO
+  Hildon::Note dialog(Hildon::NOTE_TYPE_INFORMATION, ex.what());
+#else
+  Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Internal error")), true, Gtk::MESSAGE_WARNING );
+  dialog.set_secondary_text(ex.what());
+  //TODO: dialog.set_transient_for(*get_application());
+#endif
+  dialog.run();
+}
+
 bool Base_DB::handle_error()
 {
   return ConnectionPool::handle_error();
@@ -176,7 +208,17 @@ Glib::RefPtr<Gnome::Gda::DataModel> Base_DB::query_execute(const Glib::ustring&
 
   //TODO: Bakery::BusyCursor busy_cursor(get_app_window());
 
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   sharedptr<SharedConnection> sharedconnection = connect_to_server();
+#else
+  std::auto_ptr<ExceptionConnection> error;
+  sharedptr<SharedConnection> sharedconnection = connect_to_server(0, error);
+  if(error.get())
+  {
+    g_warning("Base_DB::query_execute failed (query was: %s): %s", strQuery.c_str(), error->what());
+    // TODO: Rethrow? 
+  }
+#endif
   if(sharedconnection)
   {
     Glib::RefPtr<Gnome::Gda::Connection> gda_connection = sharedconnection->get_gda_connection();
@@ -184,17 +226,48 @@ Glib::RefPtr<Gnome::Gda::DataModel> Base_DB::query_execute(const Glib::ustring&
    const App_Glom* app = App_Glom::get_application();
    if(app && app->get_show_sql_debug())
    {
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
       try
       {
+#endif
         std::cout << "Debug: query_execute():  " << strQuery << std::endl;
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
       }
       catch(const Glib::Exception& ex)
       {
         std::cout << "Debug: query string could not be converted to std::cout: " << ex.what() << std::endl;
       }
+#endif
     }
-    
-    result = gda_connection->execute_single_command(strQuery);
+
+    // TODO: Several functions call query_execute with non-select queries.
+    // Before libgda-3.0, execute_single_command returned always a datamodel
+    // on success. In case of a successful non-select command, we therefore
+    // create an empty datamodel to return, to make clear that the function
+    // succeeded. Probably, we should introduce another
+    // query_execute_non_select in the long term. 
+    if(strQuery.compare(0, 6, "SELECT") == 0)
+    {
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+      result = gda_connection->execute_select_command(strQuery);
+#else
+      std::auto_ptr<Glib::Error> error;
+      result = gda_connection->execute_select_command(strQuery, error);
+      // Ignore error, empty datamodel is handled below
+#endif
+    }
+    else
+    {
+      std::auto_ptr<Glib::Error> error;
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+      if(gda_connection->execute_non_select_command(strQuery) != -1)
+#else
+      if(gda_connection->execute_non_select_command(strQuery, error) != -1)
+        if(error.get() == NULL)
+#endif
+          result = Gnome::Gda::DataModelArray::create(1);
+    }
+
     if(!result)
     {
       std::cerr << "Glom  Base_DB::query_execute(): Error while executing SQL" << std::endl <<
@@ -236,12 +309,23 @@ bool Base_DB::get_table_exists_in_database(const Glib::ustring& table_name) cons
 Base_DB::type_vecStrings Base_DB::get_table_names_from_database(bool ignore_system_tables) const
 {
   type_vecStrings result;
-
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   sharedptr<SharedConnection> sharedconnection = connect_to_server();
+#else
+  std::auto_ptr<ExceptionConnection> error;
+  sharedptr<SharedConnection> sharedconnection = connect_to_server(0, error);
+  // TODO: Rethrow?
+#endif
   if(sharedconnection)
   {
     Glib::RefPtr<Gnome::Gda::Connection> gda_connection = sharedconnection->get_gda_connection();
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
     Glib::RefPtr<Gnome::Gda::DataModel> data_model_tables = gda_connection->get_schema(Gnome::Gda::CONNECTION_SCHEMA_TABLES);
+#else
+    std::auto_ptr<Glib::Error> error;
+    Glib::RefPtr<Gnome::Gda::DataModel> data_model_tables = gda_connection->get_schema(Gnome::Gda::CONNECTION_SCHEMA_TABLES, error);
+    // Ignore error, data_model_tables presence is checked below
+#endif
     if(data_model_tables && (data_model_tables->get_n_columns() == 0))
     {
       std::cerr << "Base_DB_Table::get_table_names_from_database(): libgda reported 0 tables for the database." << std::endl;
@@ -255,7 +339,7 @@ Base_DB::type_vecStrings Base_DB::get_table_names_from_database(bool ignore_syst
 
         //Get the table name:
         Glib::ustring table_name;
-        if(value.get_value_type() ==  Gnome::Gda::VALUE_TYPE_STRING)
+        if(G_VALUE_TYPE(value.gobj()) ==  G_TYPE_STRING)
         {
           table_name = value.get_string();
 
@@ -378,7 +462,14 @@ Base_DB::type_vecFields Base_DB::get_fields_for_table_from_database(const Glib::
 
   //TODO: Bakery::BusyCursor busy_cursor(get_application());
 
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   sharedptr<SharedConnection> sharedconnection = connect_to_server();
+#else
+  std::auto_ptr<ExceptionConnection> error;
+  sharedptr<SharedConnection> sharedconnection = connect_to_server(0, error);
+  // TODO: Rethrow?
+#endif
+
   if(!sharedconnection)
   {
     g_warning("Base_DB::get_fields_for_table_from_database(): connection failed.");
@@ -387,11 +478,18 @@ Base_DB::type_vecFields Base_DB::get_fields_for_table_from_database(const Glib::
   {
     Glib::RefPtr<Gnome::Gda::Connection> connection = sharedconnection->get_gda_connection();
 
-    Gnome::Gda::Parameter param_table_name("name", table_name);
-    Gnome::Gda::ParameterList param_list;
-    param_list.add_parameter(param_table_name);
+    Glib::RefPtr<Gnome::Gda::Parameter> param_table_name = Gnome::Gda::Parameter::create("name", table_name);
+    //Gnome::Gda::Parameter param_table_name("name", table_name);
+    Glib::RefPtr<Gnome::Gda::ParameterList> param_list = Gnome::Gda::ParameterList::create();
+    param_list->add_parameter(param_table_name);
 
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
     Glib::RefPtr<Gnome::Gda::DataModel> data_model_fields = connection->get_schema(Gnome::Gda::CONNECTION_SCHEMA_FIELDS, param_list);
+#else
+    std::auto_ptr<Glib::Error> error;
+    Glib::RefPtr<Gnome::Gda::DataModel> data_model_fields = connection->get_schema(Gnome::Gda::CONNECTION_SCHEMA_FIELDS, param_list, error);
+    // Ignore error, data_model_fields presence is checked below
+#endif
 
     if(!data_model_fields)
     {
@@ -411,22 +509,22 @@ Base_DB::type_vecFields Base_DB::get_fields_for_table_from_database(const Glib::
       guint rows_count = data_model_fields->get_n_rows();
       while(row < rows_count)
       {
-        Gnome::Gda::FieldAttributes field_info;
+        Glib::RefPtr<Gnome::Gda::Column> field_info = Gnome::Gda::Column::create();
 
         //Get the field name:
         Gnome::Gda::Value value_name = data_model_fields->get_value_at(DATAMODEL_FIELDS_COL_NAME, row);
-        if(value_name.get_value_type() ==  Gnome::Gda::VALUE_TYPE_STRING)
+        if(value_name.get_value_type() ==  G_TYPE_STRING)
         {
           if(value_name.get_string().empty())
             g_warning("Base_DB::get_fields_for_table_from_database(): value_name is empty.");
 
-          field_info.set_name( value_name.get_string() ); //TODO: get_string() is a dodgy choice. murrayc.
+          field_info->set_name( value_name.get_string() ); //TODO: get_string() is a dodgy choice. murrayc.
         }
 
         //Get the field type:
-        //This is a string representation of the type, so we need to discover the Gnome::Gda::ValueType for it:
+        //This is a string representation of the type, so we need to discover the GType for it:
         Gnome::Gda::Value value_fieldtype = data_model_fields->get_value_at(DATAMODEL_FIELDS_COL_TYPE, row);
-        if(value_fieldtype.get_value_type() ==  Gnome::Gda::VALUE_TYPE_STRING)
+        if(value_fieldtype.get_value_type() ==  G_TYPE_STRING)
         {
            Glib::ustring schema_type_string = value_fieldtype.get_string();
            if(!schema_type_string.empty())
@@ -435,8 +533,8 @@ Base_DB::type_vecFields Base_DB::get_fields_for_table_from_database(const Glib::
              const FieldTypes* pFieldTypes = connection_pool->get_field_types();
              if(pFieldTypes)
              {
-               Gnome::Gda::ValueType gdatype = pFieldTypes->get_gdavalue_for_schema_type_string(schema_type_string);
-               field_info.set_gdatype(gdatype);
+               GType gdatype = pFieldTypes->get_gdavalue_for_schema_type_string(schema_type_string);
+               field_info->set_g_type(gdatype);
              }
            }
         }
@@ -444,33 +542,33 @@ Base_DB::type_vecFields Base_DB::get_fields_for_table_from_database(const Glib::
         //Get the default value:
         /* libgda does not return this correctly yet. TODO: check bug http://bugzilla.gnome.org/show_bug.cgi?id=143576
         Gnome::Gda::Value value_defaultvalue = data_model_fields->get_value_at(DATAMODEL_FIELDS_COL_DEFAULTVALUE, row);
-        if(value_defaultvalue.get_value_type() ==  Gnome::Gda::VALUE_TYPE_STRING)
-          field_info.set_default_value(value_defaultvalue);
+        if(value_defaultG_VALUE_TYPE(value.gobj()) ==  G_TYPE_STRING)
+          field_info->set_default_value(value_defaultvalue);
         */
 
         //Get whether it can be null:
         Gnome::Gda::Value value_notnull = data_model_fields->get_value_at(DATAMODEL_FIELDS_COL_NOTNULL, row);
-        if(value_notnull.get_value_type() ==  Gnome::Gda::VALUE_TYPE_BOOLEAN)
-          field_info.set_allow_null(value_notnull.get_bool());
+        if(value_notnull.get_value_type() ==  G_TYPE_BOOLEAN)
+          field_info->set_allow_null(value_notnull.get_boolean());
 
 
         //Get whether it is a primary key:
         Gnome::Gda::Value value_primarykey = data_model_fields->get_value_at(DATAMODEL_FIELDS_COL_PRIMARYKEY, row);
-        if(value_primarykey.get_value_type() ==  Gnome::Gda::VALUE_TYPE_BOOLEAN)
-          field_info.set_primary_key( value_primarykey.get_bool() );
+        if(value_primarykey.get_value_type() ==  G_TYPE_BOOLEAN)
+          field_info->set_primary_key( value_primarykey.get_boolean() );
 
         //Get whether it is unique
         Gnome::Gda::Value value_unique = data_model_fields->get_value_at(DATAMODEL_FIELDS_COL_UNIQUEINDEX, row);
-        if(value_unique.get_value_type() ==  Gnome::Gda::VALUE_TYPE_BOOLEAN)
-          field_info.set_unique_key( value_unique.get_bool() );
-        else if(field_info.get_primary_key()) //All primaries keys are unique, so force this.
-          field_info.set_unique_key();
+        if(value_unique.get_value_type() ==  G_TYPE_BOOLEAN)
+          field_info->set_unique_key( value_unique.get_boolean() );
+        else if(field_info->get_primary_key()) //All primaries keys are unique, so force this.
+          field_info->set_unique_key();
 
         //Get whether it is autoincrements
         /* libgda does not provide this yet.
         Gnome::Gda::Value value_autoincrement = data_model_fields->get_value_at(DATAMODEL_FIELDS_COL_AUTOINCREMENT, row);
-        if(value_autoincrement.get_value_type() ==  Gnome::Gda::VALUE_TYPE_BOOLEAN)
-          field_info.set_auto_increment( value_autoincrement.get_bool() );
+        if(value_autoincrement.get_value_type() ==  G_TYPE_BOOLEAN)
+          field_info->set_auto_increment( value_autoincrement.get_bool() );
         */
 
         sharedptr<Field> field(new Field()); //TODO: Get glom-specific information from the document?
@@ -521,19 +619,19 @@ Base_DB::type_vecFields Base_DB::get_fields_for_table(const Glib::ustring& table
 
       if(iterFindDatabase != fieldsDatabase.end() ) //Ignore fields that don't exist in the database anymore.
       {
-        Gnome::Gda::FieldAttributes field_info_document = field->get_field_info();
+        Glib::RefPtr<Gnome::Gda::Column> field_info_document = field->get_field_info();
 
         //Update the Field information that _might_ have changed in the database.
-        Gnome::Gda::FieldAttributes field_info = (*iterFindDatabase)->get_field_info();
+        Glib::RefPtr<Gnome::Gda::Column> field_info = (*iterFindDatabase)->get_field_info();
 
         //libgda does not tell us whether the field is auto_incremented, so we need to get that from the document.
-        field_info.set_auto_increment( field_info_document.get_auto_increment() );
+        field_info->set_auto_increment( field_info_document->get_auto_increment() );
 
         //libgda does not tell us whether the field is auto_incremented, so we need to get that from the document.
-        field_info.set_primary_key( field_info_document.get_primary_key() );
+        field_info->set_primary_key( field_info_document->get_primary_key() );
 
         //libgda does yet tell us correct default_value information so we need to get that from the document.
-        field_info.set_default_value( field_info_document.get_default_value() );
+        field_info->set_default_value( field_info_document->get_default_value() );
 
         field->set_field_info(field_info);
 
@@ -561,7 +659,9 @@ Base_DB::type_vecFields Base_DB::get_fields_for_table(const Glib::ustring& table
 
 bool Base_DB::add_standard_tables() const
 {
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
+#endif // GLIBMM_EXCEPTIONS_ENABLED
   {
     Document_Glom::type_vecFields pref_fields;
     sharedptr<TableInfo> prefs_table_info = Document_Glom::create_table_system_preferences(pref_fields);
@@ -640,11 +740,18 @@ bool Base_DB::add_standard_tables() const
       return false;
     }
   }
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+  catch(const Glib::Exception& ex)
+  {
+    std::cerr << "Base_DB::add_standard_tables(): caught exception: " << ex.what() << std::endl;
+    return false;
+  }
   catch(const std::exception& ex)
   {
     std::cerr << "Base_DB::add_standard_tables(): caught exception: " << ex.what() << std::endl;
     return false;
   }
+#endif // GLIBMM_EXCEPTIONS_ENABLED
 }
 
 bool Base_DB::add_standard_groups()
@@ -836,7 +943,9 @@ SystemPrefs Base_DB::get_database_preferences() const
       + Glib::ustring(optional_org_logo ? ", \"" GLOM_STANDARD_TABLE_PREFS_TABLE_NAME "\".\"" GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_LOGO "\"" : "") +
       " FROM \"" GLOM_STANDARD_TABLE_PREFS_TABLE_NAME "\"";
 
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
+#endif
   {
     Glib::RefPtr<Gnome::Gda::DataModel> datamodel = query_execute(sql_query);
     if(datamodel && (datamodel->get_n_rows() != 0))
@@ -856,10 +965,16 @@ SystemPrefs Base_DB::get_database_preferences() const
 
     }
   }
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+  catch(const Glib::Exception& ex)
+  {
+    std::cerr << "Base_DB::get_database_preferences(): exception: " << ex.what() << std::endl;
+  }
   catch(const std::exception& ex)
   {
     std::cerr << "Base_DB::get_database_preferences(): exception: " << ex.what() << std::endl;
   }
+#endif // GLIBMM_EXCEPTIONS_ENABLED
 
   return result;
 }
@@ -891,14 +1006,22 @@ void Base_DB::set_database_preferences(const SystemPrefs& prefs)
       "\"" GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_POSTCODE "\" = '" + prefs.m_org_address_postcode + "'"
       " WHERE \"" GLOM_STANDARD_TABLE_PREFS_FIELD_ID "\" = 1";
 
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
     try
+#endif // GLIBMM_EXCEPTIONS_ENABLED
     {
       Glib::RefPtr<Gnome::Gda::DataModel> datamodel = query_execute(sql_query);
     }
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+    catch(const Glib::Exception& ex)
+    {
+      std::cerr << "Base_DB::set_database_preferences(): exception: " << ex.what() << std::endl;
+    }
     catch(const std::exception& ex)
     {
       std::cerr << "Base_DB::set_database_preferences(): exception: " << ex.what() << std::endl;
     }
+#endif // GLIBMM_EXCEPTIONS_ENABLED
 
     //Set some information in the document too, so we can use it to recreate the database:
     get_document()->set_database_title(prefs.m_name);
@@ -918,8 +1041,8 @@ bool Base_DB::create_table_with_default_fields(const Glib::ustring& table_name)
   field_primary_key->set_primary_key();
   field_primary_key->set_auto_increment();
 
-  Gnome::Gda::FieldAttributes field_info = field_primary_key->get_field_info();
-  field_info.set_allow_null(false);
+  Glib::RefPtr<Gnome::Gda::Column> field_info = field_primary_key->get_field_info();
+  field_info->set_allow_null(false);
   field_primary_key->set_field_info(field_info);
 
   field_primary_key->set_glom_type(Field::TYPE_NUMERIC);
@@ -955,7 +1078,7 @@ bool Base_DB::create_table_with_default_fields(const Glib::ustring& table_name)
     //query_execute( "INSERT INTO \"" + table_name + "\" VALUES (0)" );
 
     //PostgresSQL:
-    //query_execute( "CREATE TABLE " + table_name + " (\"" + primary_key_name + "\" serial NOT NULL  PRIMARY KEY)" );
+    //query_execute( "CREATE TABLE \"" + table_name + "\" (\"" + primary_key_name + "\" serial NOT NULL  PRIMARY KEY)" );
 
     //query_execute( "CREATE TABLE \"" + table_name + "\" (" +
     //  field_primary_key->get_name() + " numeric NOT NULL  PRIMARY KEY," + 
@@ -1005,8 +1128,8 @@ bool Base_DB::create_table(const sharedptr<const TableInfo>& table_info, const D
 
     //The field has no gda type, so we set that:
     //This usually comes from the database, but that's a bit strange.
-    Gnome::Gda::FieldAttributes info = field->get_field_info();
-    info.set_gdatype( Field::get_gda_type_for_glom_type(field->get_glom_type()) );
+    Glib::RefPtr<Gnome::Gda::Column> info = field->get_field_info();
+    info->set_g_type( Field::get_gda_type_for_glom_type(field->get_glom_type()) );
     field->set_field_info(info); //TODO_Performance
 
     Glib::ustring sql_field_description = "\"" + field->get_name() + "\" " + field->get_sql_type();
@@ -1029,7 +1152,7 @@ bool Base_DB::create_table(const sharedptr<const TableInfo>& table_info, const D
   //Actually create the table
   try
   {
-    Glib::RefPtr<Gnome::Gda::DataModel> data_model = query_execute( "CREATE TABLE \"" + table_info->get_name() + "\" (" + sql_fields + ")" );
+    Glib::RefPtr<Gnome::Gda::DataModel> data_model = query_execute( "CREATE TABLE \"" + table_info->get_name() + "\" (" + sql_fields + ");" );
     if(!data_model)
       table_creation_succeeded = false;
     else
@@ -1103,7 +1226,9 @@ bool Base_DB::insert_example_data(const Glib::ustring& table_name) const
 
   for(type_vecStrings::const_iterator iter = vec_rows.begin(); iter != vec_rows.end(); ++iter)
   {
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
     try
+#endif // GLIBMM_EXCEPTIONS_ENABLED
     {
       //Check that the row contains the correct number of columns.
       //This check will slow this down, but it seems useful:
@@ -1120,9 +1245,14 @@ bool Base_DB::insert_example_data(const Glib::ustring& table_name) const
           {
             const Glib::ustring strQuery = "INSERT INTO \"" + table_name + "\" (" + strNames + ") VALUES (" + row_data + ")";
             //std::cout << "debug: before query: " << strQuery << std::endl;
-            query_execute(strQuery); //TODO: Test result.
-            //std::cout << "debug: after query: " << strQuery << std::endl;
-            insert_succeeded = true;
+            if(query_execute(strQuery))
+              //std::cout << "debug: after query: " << strQuery << std::endl;
+              insert_succeeded = true;
+	    else
+            {
+              insert_succeeded = false;
+              break;
+            }
           }
           else
           {
@@ -1135,6 +1265,7 @@ bool Base_DB::insert_example_data(const Glib::ustring& table_name) const
         }
       }
     }
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
     catch(const ExceptionConnection& ex)
     {
       std::cerr << "debug: Base_DB::insert_example_data(): query_execute() failed: " << ex.what() << std::endl;
@@ -1142,6 +1273,7 @@ bool Base_DB::insert_example_data(const Glib::ustring& table_name) const
       insert_succeeded = false;
       break;
     }
+#endif
   }
 
   for(Document_Glom::type_vecFields::const_iterator iter = vec_fields.begin(); iter != vec_fields.end(); ++iter)
@@ -1162,38 +1294,52 @@ sharedptr<LayoutItem_Field> Base_DB::offer_field_list(const sharedptr<const Layo
 {
   sharedptr<LayoutItem_Field> result;
 
+  Glib::RefPtr<Gnome::Glade::Xml> refXml;
+
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
-    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_choose_field");
-
-    Dialog_ChooseField* dialog = 0;
-    refXml->get_widget_derived("dialog_choose_field", dialog);
-
-    if(dialog)
-    {
-      if(transient_for)
-        dialog->set_transient_for(*transient_for);
-
-      dialog->set_document(get_document(), table_name, start_field);
-      //TODO: dialog->set_transient_for(*get_app_window());
-      const int response = dialog->run();
-      if(response == Gtk::RESPONSE_OK)
-      {
-        //Get the chosen field:
-        result = dialog->get_field_chosen();
-      }
-
-      delete dialog;
-    }
+    refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_choose_field");
   }
   catch(const Gnome::Glade::XmlError& ex)
   {
     std::cerr << ex.what() << std::endl;
+    return result;
+  }
+#else
+  std::auto_ptr<Gnome::Glade::XmlError> error;
+  refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_choose_field", "", error);
+  if(error.get())
+  {
+    std::cerr << error->what() << std::endl;
+    return result;
+  }
+#endif
+
+  Dialog_ChooseField* dialog = 0;
+  refXml->get_widget_derived("dialog_choose_field", dialog);
+
+  if(dialog)
+  {
+    if(transient_for)
+      dialog->set_transient_for(*transient_for);
+
+    dialog->set_document(get_document(), table_name, start_field);
+    //TODO: dialog->set_transient_for(*get_app_window());
+    const int response = dialog->run();
+    if(response == Gtk::RESPONSE_OK)
+    {
+      //Get the chosen field:
+      result = dialog->get_field_chosen();
+    }
+
+    delete dialog;
   }
 
   return result;
 }
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
 sharedptr<LayoutItem_Field> Base_DB::offer_field_formatting(const sharedptr<const LayoutItem_Field>& start_field, const Glib::ustring& table_name, Gtk::Window* transient_for)
 {
   sharedptr<LayoutItem_Field> result;
@@ -1339,6 +1485,7 @@ sharedptr<LayoutItem_Notebook> Base_DB::offer_notebook(const sharedptr<LayoutIte
 
   return result;
 }
+#endif // !GLOM_ENABLE_CLIENT_ONLY
 
 void Base_DB::fill_full_field_details(const Glib::ustring& parent_table_name, sharedptr<LayoutItem_Field>& layout_item)
 {
@@ -1351,11 +1498,18 @@ void Base_DB::fill_full_field_details(const Glib::ustring& parent_table_name, sh
 //static
 bool Base_DB::show_warning_no_records_found(Gtk::Window& transient_for)
 {
+  Glib::ustring message = _("Your find criteria did not match any records in the table.");
+
+#ifdef GLOM_ENABLE_MAEMO
+  Hildon::Note dialog(Hildon::NOTE_TYPE_CONFIRMATION_BUTTON, transient_for, message);
+#else
   Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("No Records Found")), true, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_NONE);
+  dialog.set_secondary_text(message);
+  dialog.set_transient_for(transient_for);
+#endif
+
   dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
   dialog.add_button(_("New Find"), Gtk::RESPONSE_OK);
-  dialog.set_secondary_text(_("Your find criteria did not match any records in the table."));
-  dialog.set_transient_for(transient_for);
 
   const bool find_again = (dialog.run() == Gtk::RESPONSE_OK);
   return find_again;
@@ -1784,7 +1938,15 @@ void Base_DB::calculate_field(const LayoutFieldInRecord& field_in_record)
         if(field)
         {
           //We need the connection when we run the script, so that the script may use it.
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
           sharedptr<SharedConnection> sharedconnection = connect_to_server(0 /* parent window */);
+#else
+	  std::auto_ptr<ExceptionConnection> error;
+	  sharedptr<SharedConnection> sharedconnection = connect_to_server(0 /* parent window */, error);
+	  // TODO: Rethrow?
+#endif
+
+	  g_assert(sharedconnection);
 
           refCalcProgress.m_value = glom_evaluate_python_function_implementation(field->get_glom_type(), field->get_calculation(), field_values, get_document(), field_in_record.m_table_name, sharedconnection->get_gda_connection());
 
@@ -1853,7 +2015,7 @@ Base_DB::type_map_fields Base_DB::get_record_field_values_for_calculation(const
 
           //Never give a NULL-type value to the python calculation for types that don't use them:
           //to prevent errors:
-          if(value.get_value_type() == Gnome::Gda::VALUE_TYPE_NULL)
+          if(value.is_null())
             value = Conversions::get_empty_value(field->get_glom_type());
 
           field_values[field->get_name()] = value;
@@ -1926,7 +2088,9 @@ bool Base_DB::set_field_value_in_database(const LayoutFieldInRecord& layoutfield
 
     //std::cout << "debug: set_field_value_in_database(): " << std::endl << "  " << strQuery << std::endl;
 
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
     try
+#endif
     {
       Glib::RefPtr<Gnome::Gda::DataModel> datamodel = query_execute(strQuery, parent_window);  //TODO: Handle errors
       if(!datamodel)
@@ -1935,11 +2099,18 @@ bool Base_DB::set_field_value_in_database(const LayoutFieldInRecord& layoutfield
         return false; //failed.
       }
     }
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+    catch(const Glib::Exception& ex)
+    {
+      handle_error(ex);
+      return false;
+    }
     catch(const std::exception& ex)
     {
       handle_error(ex);
       return false;
     }
+#endif
 
     //Get-and-set values for lookup fields, if this field triggers those relationships:
     do_lookups(layoutfield_in_record, row, field_value);
