@@ -22,20 +22,23 @@
 #define GLOM_UTILITYWIDGETS_FLOWTABLEWITHFIELDS_H
 
 #include "flowtable.h"
-#include "../data_structure/layout/layoutgroup.h"
-#include "../data_structure/layout/layoutitem_field.h"
-#include "../data_structure/layout/layoutitem_notebook.h"
-#include "../data_structure/layout/layoutitem_portal.h"
-#include "../data_structure/layout/layoutitem_button.h"
-#include "../data_structure/layout/layoutitem_text.h"
-#include "../data_structure/field.h"
-#include "../document/document_glom.h"
+#include <glom/libglom/data_structure/layout/layoutgroup.h>
+#include <glom/libglom/data_structure/layout/layoutitem_field.h>
+#include <glom/libglom/data_structure/layout/layoutitem_notebook.h>
+#include <glom/libglom/data_structure/layout/layoutitem_portal.h>
+#include <glom/libglom/data_structure/layout/layoutitem_button.h>
+#include <glom/libglom/data_structure/layout/layoutitem_text.h>
+#include <glom/libglom/data_structure/field.h>
+#include <glom/libglom/document/document_glom.h>
 #include "../mode_data/box_data_list_related.h"
 #include "layoutwidgetbase.h"
 #include "../mode_data/treestore_layout.h" //Forthe enum.
 #include <map>
 #include <list>
 
+namespace Glom
+{
+
 class DataWidget;
 
 class FlowTableWithFields
@@ -154,7 +157,7 @@ protected:
     sharedptr<const LayoutItem_Field> m_field; //Store the field information so we know the title, ID, and type.
     Glib::ustring m_group;
 
-    Gtk::Alignment* m_first;
+    Gtk::Label* m_first;
     DataWidget* m_second;
     Gtk::CheckButton* m_checkbutton; //Used instead of first and second if it's a bool.
   };
@@ -199,5 +202,6 @@ protected:
   type_signal_script_button_clicked m_signal_script_button_clicked;
 };
 
+} //namespace Glom
 
 #endif //GLOM_UTILITYWIDGETS_FLOWTABLEWITHFIELDS_H
